@@ -1,3 +1,6 @@
+import sys
+
+from . import typing
 from ._connection import (
     FakeRedis,
     FakeStrictRedis,
@@ -9,11 +12,16 @@ from .aioredis import (
     FakeConnection as FakeAsyncConnection,
 )
 
-try:
-    from importlib import metadata
-except ImportError:  # for Python < 3.8
-    import importlib_metadata as metadata  # type: ignore
-__version__ = metadata.version("fakeredis")
+if sys.version_info >= (3, 11):
+    from ._tcp_server import TcpFakeServer
+else:
+
+    class TcpFakeServer:
+        def __init__(self, *args, **kwargs):
+            raise NotImplementedError("TcpFakeServer is only available in Python 3.11+")
+
+
+__version__ = typing.lib_version
 __author__ = "Daniel Moran"
 __maintainer__ = "Daniel Moran"
 __email__ = "daniel@moransoftware.ca"
@@ -28,4 +36,19 @@ __all__ = [
     "FakeConnection",
     "FakeAsyncRedis",
     "FakeAsyncConnection",
+    "TcpFakeServer",
 ]
+
+try:
+    import valkey  # noqa: F401
+    from ._valkey import FakeValkey, FakeAsyncValkey, FakeStrictValkey  # noqa: F401
+
+    __all__.extend(
+        [
+            "FakeValkey",
+            "FakeAsyncValkey",
+            "FakeStrictValkey",
+        ]
+    )
+except ImportError:
+    pass
