@@ -3,18 +3,18 @@ import threading
 import time
 import weakref
 from collections import defaultdict
-from typing import Dict, Tuple, Any, List, Optional, Union, Literal
+from typing import Dict, Tuple, Any, List, Optional, Union
 
+import redis
+
+from fakeredis.model import AccessControlList
 from fakeredis._helpers import Database, FakeSelector
+from fakeredis.typing import VersionType, ServerType
 
 LOGGER = logging.getLogger("fakeredis")
 
-VersionType = Union[Tuple[int, ...], int, str]
 
-ServerType = Literal["redis", "dragonfly", "valkey"]
-
-
-def _create_version(v: VersionType) -> Tuple[int, ...]:
+def _create_version(v: Union[Tuple[int, ...], int, str]) -> VersionType:
     if isinstance(v, tuple):
         return v
     if isinstance(v, int):
@@ -25,10 +25,30 @@ def _create_version(v: VersionType) -> Tuple[int, ...]:
     return v
 
 
+def _version_to_str(v: VersionType) -> str:
+    if isinstance(v, tuple):
+        return ".".join(str(x) for x in v)
+    return str(v)
+
+
 class FakeServer:
     _servers_map: Dict[str, "FakeServer"] = dict()
 
-    def __init__(self, version: VersionType = (7,), server_type: ServerType = "redis") -> None:
+    def __init__(
+        self,
+        version: VersionType = (7,),
+        server_type: ServerType = "redis",
+        config: Optional[Dict[bytes, bytes]] = None,
+    ) -> None:
+        """Initialize a new FakeServer instance.
+        :param version: The version of the server (e.g. 6, 7.4, "7.4.1", can also be a tuple)
+        :param server_type: The type of server (redis, dragonfly, valkey)
+        :param config: A dictionary of configuration options.
+
+        Configuration options:
+        - `requirepass`: The password required to authenticate to the server.
+        - `aclfile`: The path to the ACL file.
+        """
         self.lock = threading.Lock()
         self.dbs: Dict[int, Database] = defaultdict(lambda: Database(self.lock))
         # Maps channel/pattern to a weak set of sockets
@@ -38,24 +58,32 @@ class FakeServer:
         self.lastsave: int = int(time.time())
         self.connected = True
         # List of weakrefs to sockets that are being closed lazily
+        self.sockets: List[Any] = []
         self.closed_sockets: List[Any] = []
-        self.version: Tuple[int, ...] = _create_version(version)
+        self.version: VersionType = _create_version(version)
         if server_type not in ("redis", "dragonfly", "valkey"):
             raise ValueError(f"Unsupported server type: {server_type}")
         self.server_type: str = server_type
+        self.config: Dict[bytes, bytes] = config or dict()
+        self.acl: AccessControlList = AccessControlList()
 
     @staticmethod
-    def get_server(key: str, version: VersionType, server_type: str) -> "FakeServer":
-        return FakeServer._servers_map.setdefault(key, FakeServer(version=version, server_type=server_type))
+    def get_server(key: str, version: VersionType, server_type: ServerType) -> "FakeServer":
+        if key not in FakeServer._servers_map:
+            FakeServer._servers_map[key] = FakeServer(version=version, server_type=server_type)
+        return FakeServer._servers_map[key]
 
 
 class FakeBaseConnectionMixin(object):
-    def __init__(self, *args: Any, version: VersionType = (7, 0), server_type: str = "redis", **kwargs: Any) -> None:
+    def __init__(
+        self, *args: Any, version: VersionType = (7, 0), server_type: ServerType = "redis", **kwargs: Any
+    ) -> None:
         self.client_name: Optional[str] = None
         self.server_key: str
         self._sock = None
         self._selector: Optional[FakeSelector] = None
         self._server = kwargs.pop("server", None)
+        self._client_class = kwargs.pop("client_class", redis.Redis)
         self._lua_modules = kwargs.pop("lua_modules", set())
         path = kwargs.pop("path", None)
         connected = kwargs.pop("connected", True)
@@ -65,7 +93,7 @@ class FakeBaseConnectionMixin(object):
             else:
                 host, port = kwargs.get("host"), kwargs.get("port")
                 self.server_key = f"{host}:{port}"
-            self.server_key += f":{server_type}:v{version}"
+            self.server_key += f":{server_type}:v{_version_to_str(version)[0]}"
             self._server = FakeServer.get_server(self.server_key, server_type=server_type, version=version)
             self._server.connected = connected
         super().__init__(*args, **kwargs)
