@@ -2,11 +2,11 @@ import logging
 from dataclasses import dataclass
 from itertools import count
 from socketserver import ThreadingTCPServer, StreamRequestHandler
-from typing import BinaryIO, Dict, Tuple
+from typing import BinaryIO, Dict, Tuple, Any
 
 from fakeredis import FakeRedis
 from fakeredis import FakeServer
-from fakeredis._server import ServerType
+from fakeredis.typing import VersionType, ServerType
 
 LOGGER = logging.getLogger("fakeredis")
 LOGGER.setLevel(logging.DEBUG)
@@ -28,17 +28,15 @@ class Client:
 class Reader:
     reader: BinaryIO
 
-    def load_array(self, length: int):
-        array = [None] * length
-        for i in range(length):
-            array[i] = self.load()
-        return array
-
-    def load(self):
+    def load(self) -> Any:
         line = self.reader.readline().strip()
         match line[0:1], line[1:]:
             case b"*", length:
-                return self.load_array(int(length))
+                length = int(length)
+                array = [None] * length
+                for i in range(length):
+                    array[i] = self.load()
+                return array
             case b"$", length:
                 bulk_string = self.reader.read(int(length) + 2).strip()
                 if len(bulk_string) != int(length):
@@ -58,7 +56,7 @@ class Reader:
 class Writer:
     writer: BinaryIO
 
-    def dump(self, value, dump_bulk=False):
+    def dump(self, value: Any, dump_bulk=False) -> None:
         if isinstance(value, int):
             self.writer.write(f":{value}\r\n".encode())
         elif isinstance(value, (str, bytes)):
@@ -78,7 +76,6 @@ class Writer:
 
 
 class TCPFakeRequestHandler(StreamRequestHandler):
-
     def setup(self) -> None:
         super().setup()
         if self.client_address in self.server.clients:
@@ -116,7 +113,7 @@ class TcpFakeServer(ThreadingTCPServer):
         server_address: Tuple[str | bytes | bytearray, int],
         bind_and_activate: bool = True,
         server_type: ServerType = "redis",
-        server_version: Tuple[int, ...] = (7, 4),
+        server_version: VersionType = (8, 0),
     ):
         super().__init__(server_address, TCPFakeRequestHandler, bind_and_activate)
         self.fake_server = FakeServer(server_type=server_type, version=server_version)
