@@ -1,21 +1,23 @@
 import random
-from typing import Callable, Tuple, Any, Optional, List, Union
+from typing import Callable, Any, Optional, List, Union, Sequence
 
 from fakeredis import _msgs as msgs
 from fakeredis._commands import command, Key, Int, CommandItem
 from fakeredis._helpers import OK, SimpleError, casematch, Database, SimpleString
+from fakeredis.model import ExpiringMembersSet
+from fakeredis.typing import VersionType
 
 
 def _calc_setop(op: Callable[..., Any], stop_if_missing: bool, key: CommandItem, *keys: CommandItem) -> Any:
     if stop_if_missing and not key.value:
         return set()
     value = key.value
-    if not isinstance(value, set):
+    if not isinstance(value, ExpiringMembersSet):
         raise SimpleError(msgs.WRONGTYPE_MSG)
     ans = value.copy()
     for other in keys:
-        value = other.value if other.value is not None else set()
-        if not isinstance(value, set):
+        value = other.value if other.value is not None else ExpiringMembersSet()
+        if not isinstance(value, ExpiringMembersSet):
             raise SimpleError(msgs.WRONGTYPE_MSG)
         if stop_if_missing and not value:
             return set()
@@ -24,11 +26,8 @@ def _calc_setop(op: Callable[..., Any], stop_if_missing: bool, key: CommandItem,
 
 
 def _setop(
-        op: Callable[..., Any],
-        stop_if_missing: bool,
-        dst: Optional[CommandItem],
-        key: CommandItem,
-        *keys: CommandItem) -> Any:
+    op: Callable[..., Any], stop_if_missing: bool, dst: Optional[CommandItem], key: CommandItem, *keys: CommandItem
+) -> Any:
     """Apply one of SINTER[STORE], SUNION[STORE], SDIFF[STORE].
 
     If `stop_if_missing`, the output will be made an empty set as soon as
@@ -44,33 +43,33 @@ def _setop(
 
 
 class SetCommandsMixin:
-    _scan: Callable[..., Any]
+    _scan: Callable[[Sequence[bytes], int, bytes], List[Union[bytes, List[bytes]]]]
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         super(SetCommandsMixin, self).__init__(*args, **kwargs)
-        self.version: Tuple[int]
+        self.version: VersionType
         self._db: Database
 
-    @command((Key(set), bytes), (bytes,))
+    @command((Key(ExpiringMembersSet), bytes), (bytes,))
     def sadd(self, key: CommandItem, *members: bytes) -> int:
         old_size = len(key.value)
         key.value.update(members)
         key.updated()
         return len(key.value) - old_size
 
-    @command((Key(set),))
+    @command((Key(ExpiringMembersSet),))
     def scard(self, key: CommandItem) -> int:
         return len(key.value)
 
-    @command((Key(set),), (Key(set),))
+    @command((Key(ExpiringMembersSet),), (Key(ExpiringMembersSet),))
     def sdiff(self, *keys: CommandItem) -> Any:
         return _setop(lambda a, b: a - b, False, None, *keys)
 
-    @command((Key(), Key(set)), (Key(set),))
+    @command((Key(), Key(ExpiringMembersSet)), (Key(ExpiringMembersSet),))
     def sdiffstore(self, dst: CommandItem, *keys: CommandItem) -> Any:
         return _setop(lambda a, b: a - b, False, dst, *keys)
 
-    @command((Key(set),), (Key(set),))
+    @command((Key(ExpiringMembersSet),), (Key(ExpiringMembersSet),))
     def sinter(self, *keys: CommandItem) -> Any:
         res = _setop(lambda a, b: a & b, True, None, *keys)
         return res
@@ -87,31 +86,28 @@ class SetCommandsMixin:
             args = args[:-2]
         if numkeys != len(args):
             raise SimpleError(msgs.SYNTAX_ERROR_MSG)
-        keys = [
-            CommandItem(args[i], self._db, item=self._db.get(args[i], default=None))
-            for i in range(numkeys)
-        ]
+        keys = [CommandItem(args[i], self._db, item=self._db.get(args[i], default=None)) for i in range(numkeys)]
 
         res = _setop(lambda a, b: a & b, False, None, *keys)
         return len(res) if limit == 0 else min(limit, len(res))
 
-    @command((Key(), Key(set)), (Key(set),))
+    @command((Key(), Key(ExpiringMembersSet)), (Key(ExpiringMembersSet),))
     def sinterstore(self, dst: CommandItem, *keys: CommandItem) -> Any:
         return _setop(lambda a, b: a & b, True, dst, *keys)
 
-    @command((Key(set), bytes))
+    @command((Key(ExpiringMembersSet), bytes))
     def sismember(self, key: CommandItem, member: bytes) -> int:
         return int(member in key.value)
 
-    @command((Key(set), bytes), (bytes,))
+    @command((Key(ExpiringMembersSet), bytes), (bytes,))
     def smismember(self, key: CommandItem, *members: bytes) -> List[int]:
         return [self.sismember(key, member) for member in members]
 
-    @command((Key(set),))
+    @command((Key(ExpiringMembersSet),))
     def smembers(self, key: CommandItem) -> List[bytes]:
         return list(key.value)
 
-    @command((Key(set, 0), Key(set), bytes))
+    @command((Key(ExpiringMembersSet, 0), Key(ExpiringMembersSet), bytes))
     def smove(self, src: CommandItem, dst: CommandItem, member: bytes) -> int:
         try:
             src.value.remove(member)
@@ -123,7 +119,7 @@ class SetCommandsMixin:
             dst.updated()  # TODO: is it updated if member was already present?
             return 1
 
-    @command((Key(set),), (Int,))
+    @command((Key(ExpiringMembersSet),), (Int,))
     def spop(self, key: CommandItem, count: Optional[int] = None) -> Union[bytes, List[bytes], None]:
         if count is None:
             if not key.value:
@@ -141,7 +137,7 @@ class SetCommandsMixin:
                 key.updated()  # Inside the loop because redis special-cases count=0
             return items
 
-    @command((Key(set),), (Int,))
+    @command((Key(ExpiringMembersSet),), (Int,))
     def srandmember(self, key: CommandItem, count: Optional[int] = None) -> Union[bytes, List[bytes], None]:
         if count is None:
             if not key.value:
@@ -155,7 +151,7 @@ class SetCommandsMixin:
             items = list(key.value)
             return [random.choice(items) for _ in range(-count)]
 
-    @command((Key(set), bytes), (bytes,))
+    @command((Key(ExpiringMembersSet), bytes), (bytes,))
     def srem(self, key: CommandItem, *members: bytes) -> int:
         old_size = len(key.value)
         for member in members:
@@ -165,15 +161,15 @@ class SetCommandsMixin:
             key.updated()
         return deleted
 
-    @command((Key(set), Int), (bytes, bytes))
+    @command((Key(ExpiringMembersSet), Int), (bytes, bytes))
     def sscan(self, key: CommandItem, cursor: int, *args: bytes) -> Any:
         return self._scan(key.value, cursor, *args)
 
-    @command((Key(set),), (Key(set),))
+    @command((Key(ExpiringMembersSet),), (Key(ExpiringMembersSet),))
     def sunion(self, *keys: CommandItem) -> Any:
         return _setop(lambda a, b: a | b, False, None, *keys)
 
-    @command((Key(), Key(set)), (Key(set),))
+    @command((Key(), Key(ExpiringMembersSet)), (Key(ExpiringMembersSet),))
     def sunionstore(self, dst: CommandItem, *keys: CommandItem) -> Any:
         return _setop(lambda a, b: a | b, False, dst, *keys)
 
@@ -182,19 +178,19 @@ class SetCommandsMixin:
     # approximate and store the results in a string. Instead, it is implemented
     # on top of sets.
 
-    @command((Key(set),), (bytes,))
+    @command((Key(ExpiringMembersSet),), (bytes,))
     def pfadd(self, key: CommandItem, *elements: bytes) -> int:
         result = self.sadd(key, *elements)
         # Per the documentation:
         # - 1 if at least 1 HyperLogLog internal register was altered. 0 otherwise.
         return 1 if result > 0 else 0
 
-    @command((Key(set),), (Key(set),))
+    @command((Key(ExpiringMembersSet),), (Key(ExpiringMembersSet),))
     def pfcount(self, *keys: CommandItem) -> int:
         """Return the approximated cardinality of the set observed by the HyperLogLog at key(s)."""
         return len(self.sunion(*keys))
 
-    @command((Key(set), Key(set)), (Key(set),))
+    @command((Key(ExpiringMembersSet), Key(ExpiringMembersSet)), (Key(ExpiringMembersSet),))
     def pfmerge(self, dest: CommandItem, *sources: CommandItem) -> SimpleString:
         """Merge N different HyperLogLogs into a single one."""
         self.sunionstore(dest, *sources)
