@@ -14,7 +14,7 @@ from fakeredis import _msgs as msgs
 from fakeredis._command_args_parsing import extract_args
 from fakeredis._commands import Key, command, delete_keys, CommandItem, Int, Float
 from fakeredis._helpers import SimpleString
-from fakeredis._zset import ZSet
+from fakeredis.model import ZSet
 
 JsonType = Union[str, int, float, bool, None, Dict[str, Any], List[Any]]
 
@@ -81,10 +81,10 @@ class JSONObject:
 
 
 def _json_write_iterate(
-    method: Callable[[JsonType], Tuple[Optional[JsonType], Optional[Any], bool]],
+    method: Callable[[JsonType], Tuple[Optional[JsonType], Optional[JsonType], bool]],
     key: CommandItem,
     path_str: Union[str, bytes],
-    **kwargs: Any,
+    allow_result_none: bool = False,
 ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
     """Implement json.* write commands.
     Iterate over values with path_str in key and running method to get new value for path item.
@@ -107,11 +107,11 @@ def _json_write_iterate(
     key.update(curr_value)
 
     if len(path_str) > 1 and path_str[0] == ord(b"."):
-        if kwargs.get("allow_result_none", False):
+        if allow_result_none:
             return res[-1]
         else:
             return next(x for x in reversed(res) if x is not None)
-    if len(res) == 1 and path_str[0] != ord(b"$"):
+    if len(res) == 1 and (path_str[0] != ord(b"$") or path_str == b"."):
         return res[0]
     return res
 
@@ -124,22 +124,22 @@ def _json_read_iterate(
 ) -> Union[List[Optional[Any]], Optional[Any]]:
     path_str = args[0] if len(args) > 0 else "$"
     if key.value is None:
-        if path_str[0] == 36:
+        if path_str[0] == ord(b"$"):
             raise helpers.SimpleError(msgs.JSON_KEY_NOT_FOUND)
         else:
             return None
 
     path = _parse_jsonpath(path_str)
     found_matches = path.find(key.value)
-    if error_on_zero_matches and len(found_matches) == 0 and path_str[0] != 36:
+    if error_on_zero_matches and len(found_matches) == 0 and path_str[0] != ord(b"$"):
         raise helpers.SimpleError(msgs.JSON_PATH_NOT_FOUND_OR_NOT_STRING.format(path_str))
     res = list()
     for item in found_matches:
         res.append(method(item.value))
 
-    if path_str[0] == 46:
+    if len(path_str) > 1 and path_str[0] == ord(b"."):
         return res[0] if len(res) > 0 else None
-    if len(res) == 1 and (len(args) == 0 or (len(args) == 1 and args[0][0] == 46)):
+    if len(res) == 1 and (len(args) == 0 or path_str[0] == ord(b".")):
         return res[0]
 
     return res
@@ -170,6 +170,7 @@ class JSONCommandsMixin:
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
         self._db: helpers.Database
+        self.protocol_version: int
 
     @staticmethod
     def _get_single(
@@ -219,11 +220,12 @@ class JSONCommandsMixin:
         path = _parse_jsonpath(path_str)
         if key.value is not None and (type(key.value) is not dict) and not _path_is_root(path):
             raise helpers.SimpleError(msgs.JSON_WRONG_REDIS_TYPE)
-        old_value = path.find(key.value)
+        old_value_list = path.find(key.value)
         (nx, xx), _ = extract_args(args, ("nx", "xx"))
         if xx and nx:
             raise helpers.SimpleError(msgs.SYNTAX_ERROR_MSG)
-        if (nx and old_value) or (xx and not old_value):
+        old_value = old_value_list[0].value if len(old_value_list) > 0 else None
+        if (nx and old_value is not None) or (xx and old_value is None):
             return None
         new_value = path.update_or_create(key.value, value)
         key.update(new_value)
@@ -233,7 +235,7 @@ class JSONCommandsMixin:
         name="JSON.SET",
         fixed=(Key(), bytes, JSONObject),
         repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
+        flags=msgs.FLAG_LEAVE_EMPTY_VAL + msgs.FLAG_DO_NOT_CREATE,
     )
     def json_set(self, key: CommandItem, path_str: bytes, value: JsonType, *args: bytes) -> Optional[SimpleString]:
         """Set the JSON value at key `name` under the `path` to `obj`.
@@ -242,12 +244,7 @@ class JSONCommandsMixin:
         """
         return JSONCommandsMixin._json_set(key, path_str, value, *args)
 
-    @command(
-        name="JSON.GET",
-        fixed=(Key(),),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.GET", fixed=(Key(),), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_get(self, key: CommandItem, *args: bytes) -> Optional[bytes]:
         if key.value is None:
             return None
@@ -266,12 +263,7 @@ class JSONCommandsMixin:
             return JSONObject.encode(path_values)
         return JSONObject.encode(dict(zip(formatted_paths, path_values)))
 
-    @command(
-        name="JSON.MGET",
-        fixed=(bytes,),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.MGET", fixed=(bytes,), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_mget(self, *args: bytes) -> List[Optional[bytes]]:
         if len(args) < 2:
             raise helpers.SimpleError(msgs.WRONG_ARGS_MSG6.format("json.mget"))
@@ -281,12 +273,7 @@ class JSONCommandsMixin:
         result = [JSONObject.encode(self._get_single(key, path_str, empty_list_as_none=True)) for key in keys]
         return result
 
-    @command(
-        name="JSON.TOGGLE",
-        fixed=(Key(),),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.TOGGLE", fixed=(Key(),), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_toggle(self, key: CommandItem, *args: bytes) -> Union[List[Optional[bool]], Optional[bool]]:
         if key.value is None:
             raise helpers.SimpleError(msgs.JSON_KEY_NOT_FOUND)
@@ -311,12 +298,7 @@ class JSONCommandsMixin:
 
         return res
 
-    @command(
-        name="JSON.CLEAR",
-        fixed=(Key(),),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.CLEAR", fixed=(Key(),), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_clear(self, key: CommandItem, *args: bytes) -> int:
         if key.value is None:
             raise helpers.SimpleError(msgs.JSON_KEY_NOT_FOUND)
@@ -334,12 +316,7 @@ class JSONCommandsMixin:
         key.update(curr_value)
         return res
 
-    @command(
-        name="JSON.STRAPPEND",
-        fixed=(Key(), bytes),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.STRAPPEND", fixed=(Key(), bytes), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_strappend(
         self, key: CommandItem, path_str: bytes, *args: bytes
     ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
@@ -356,12 +333,7 @@ class JSONCommandsMixin:
 
         return _json_write_iterate(strappend, key, path_str)
 
-    @command(
-        name="JSON.ARRAPPEND",
-        fixed=(Key(), bytes),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.ARRAPPEND", fixed=(Key(), bytes), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_arrappend(
         self, key: CommandItem, path_str: bytes, *args: bytes
     ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
@@ -379,12 +351,7 @@ class JSONCommandsMixin:
 
         return _json_write_iterate(arrappend, key, path_str)
 
-    @command(
-        name="JSON.ARRINSERT",
-        fixed=(Key(), bytes, Int),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.ARRINSERT", fixed=(Key(), bytes, Int), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_arrinsert(
         self, key: CommandItem, path_str: bytes, index: int, *args: bytes
     ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
@@ -417,12 +384,7 @@ class JSONCommandsMixin:
 
         return _json_write_iterate(arrpop, key, path_str, allow_result_none=True)
 
-    @command(
-        name="JSON.ARRTRIM",
-        fixed=(Key(),),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.ARRTRIM", fixed=(Key(),), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_arrtrim(self, key: CommandItem, *args: bytes) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
         path_str: bytes = args[0] if len(args) > 0 else b"$"
         start = Int.decode(args[1]) if len(args) > 1 else 0
@@ -445,7 +407,7 @@ class JSONCommandsMixin:
         name="JSON.NUMINCRBY",
         fixed=(Key(), bytes, Float),
         repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
+        flags=msgs.FLAG_LEAVE_EMPTY_VAL + msgs.FLAG_SKIP_CONVERT_TO_RESP2,
     )
     def json_numincrby(
         self, key: CommandItem, path_str: bytes, inc_by: float, *_: bytes
@@ -457,13 +419,13 @@ class JSONCommandsMixin:
             else:
                 return None, None, False
 
-        return _json_write_iterate(numincrby, key, path_str)
+        return self._resp3_wrapping_list(_json_write_iterate(numincrby, key, path_str))
 
     @command(
         name="JSON.NUMMULTBY",
         fixed=(Key(), bytes, Float),
         repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
+        flags=msgs.FLAG_LEAVE_EMPTY_VAL + msgs.FLAG_SKIP_CONVERT_TO_RESP2,
     )
     def json_nummultby(
         self, key: CommandItem, path_str: bytes, mult_by: float, *_: bytes
@@ -475,15 +437,10 @@ class JSONCommandsMixin:
             else:
                 return None, None, False
 
-        return _json_write_iterate(nummultby, key, path_str)
+        return self._resp3_wrapping_list(_json_write_iterate(nummultby, key, path_str))
 
     # Read operations
-    @command(
-        name="JSON.ARRINDEX",
-        fixed=(Key(), bytes, bytes),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.ARRINDEX", fixed=(Key(), bytes, bytes), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_arrindex(
         self, key: CommandItem, path_str: bytes, encoded_value: bytes, *args: bytes
     ) -> Union[List[Optional[JsonType]], Optional[JsonType]]:
@@ -506,7 +463,7 @@ class JSONCommandsMixin:
             except StopIteration:
                 return -1
 
-        return _json_read_iterate(check_index, key, path_str, *args, error_on_zero_matches=True)
+        return _json_read_iterate(check_index, key, path_str, error_on_zero_matches=True)
 
     @command(name="JSON.STRLEN", fixed=(Key(),), repeat=(bytes,))
     def json_strlen(self, key: CommandItem, *args: bytes) -> Union[List[Optional[int]], Optional[int]]:
@@ -520,14 +477,17 @@ class JSONCommandsMixin:
     def json_objlen(self, key: CommandItem, *args: bytes) -> Union[List[Optional[int]], Optional[int]]:
         return _json_read_iterate(lambda val: len(val) if type(val) is dict else None, key, *args)
 
-    @command(
-        name="JSON.TYPE",
-        fixed=(Key(),),
-        repeat=(bytes,),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    def _resp3_wrapping_list(self, res: Any, wrap_list: bool = False) -> Any:
+        if self.protocol_version == 2:
+            return res
+        if isinstance(res, list) and not wrap_list:
+            return res
+        return [res]
+
+    @command(name="JSON.TYPE", fixed=(Key(),), repeat=(bytes,), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_type(self, key: CommandItem, *args: bytes) -> Union[List[Optional[bytes]], Optional[bytes]]:
-        return _json_read_iterate(lambda val: self.TYPE_NAMES.get(type(val), None), key, *args)
+        res = _json_read_iterate(lambda val: self.TYPE_NAMES.get(type(val), None), key, *args)
+        return self._resp3_wrapping_list(res, wrap_list=True)
 
     @command(name="JSON.OBJKEYS", fixed=(Key(),), repeat=(bytes,))
     def json_objkeys(self, key: CommandItem, *args: bytes) -> Union[List[Optional[bytes]], Optional[bytes]]:
@@ -535,12 +495,7 @@ class JSONCommandsMixin:
             lambda val: [i.encode() for i in val.keys()] if type(val) is dict else None, key, *args
         )
 
-    @command(
-        name="JSON.MSET",
-        fixed=(),
-        repeat=(Key(), bytes, JSONObject),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.MSET", fixed=(), repeat=(Key(), bytes, JSONObject), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_mset(self, *args: Any) -> SimpleString:
         if len(args) < 3 or len(args) % 3 != 0:
             raise helpers.SimpleError(msgs.WRONG_ARGS_MSG6.format("json.mset"))
@@ -549,12 +504,7 @@ class JSONCommandsMixin:
             JSONCommandsMixin._json_set(key, path_str, value)
         return helpers.OK
 
-    @command(
-        name="JSON.MERGE",
-        fixed=(Key(), bytes, JSONObject),
-        repeat=(),
-        flags=msgs.FLAG_LEAVE_EMPTY_VAL,
-    )
+    @command(name="JSON.MERGE", fixed=(Key(), bytes, JSONObject), repeat=(), flags=msgs.FLAG_LEAVE_EMPTY_VAL)
     def json_merge(self, key: CommandItem, path_str: bytes, value: JsonType) -> SimpleString:
         path: JSONPath = _parse_jsonpath(path_str)
         if key.value is not None and (type(key.value) is not dict) and not _path_is_root(path):
